//Library to easily create a reactor

//ed          External diameter of the reactor (mm)
//id          Internal diameter of the reactor (mm)
//h           height of the reactor (mm)
//flat_bot    (bool) If true, the bottom of the internal chamber will be flat. I false, the bottom will be   round. Default set to false
//open        (bool) If true, the top of the reactor will be open. Default set to false
//adaptor_top (bool) If true and 'open' is false, create an input at the top of the reactor. Default set to true
//port_bot    (bool) If true, create an output siphon at the bottom of the reactor. Default set to true
//adaptor_bot (bool) If true, create space for an adaptor at the bottom. Default set to false


use <lib_pipe.scad>
use <lib_bend_pipe.scad>

module reactor(ed, id, h, flat_bot=false, open=false, adaptor_top=true, port_bot=true, adaptor_bot=false)
{
    //Thickness of the wall
    WALL = (ed - id) / 2;

    //Thickness of the wall for the top connector
    WALL_CON = 3;

    //Diameter of the canula
    D_CANULA = 3;

    //Spacing canula: from the bottom of the hemisphere to the perpendicular tubing
    S_CANULA = 1;

    //Diameter and length for the connectors
    D_CON = 5;
    L_CON = 5;

    //Length of connector at the bottom
    L_CON_BOT = 10;

    //Main tube
    translate([0, 0, id / 2 + WALL]) tube(ed=ed, id=id, h=h - id - 2 * WALL);

    //Bottom
    if (flat_bot == false)
    {
        //Round bottom, no port
        if (port_bot == false)
        {
            //Bottom, hemisphere inside
            difference()
            {
                cylinder(d=ed, h=id / 2 + WALL);
                translate([0, 0, id / 2 + WALL]) sphere(d=id);
            }
        }
        //Round bottom, port
        else
        {
            difference()
            {
                cylinder(d=ed, h=id / 2 + WALL + D_CANULA + D_CON / 2 + S_CANULA);
                translate([0, 0, id / 2 + WALL + D_CANULA + D_CON / 2 + S_CANULA]) sphere(d=id);

                //Adaptor tube
                if (adaptor_bot == true)
                {
                    translate([0, -ed / 2 + L_CON_BOT, D_CON / 2 + WALL]) rotate([90, 0, 0]) cylinder(d=D_CON, h=L_CON_BOT);
                }

                //Perpendicular tube, canula
                translate([0, -D_CANULA, D_CON / 2 + WALL]) rotate([90, 0, 0]) cylinder(d=D_CANULA, h=ed / 2);
                echo("Reactor, height for bottom port:", D_CON / 2 + WALL);

                //Bend pipe, canula
                translate([0, -D_CANULA, WALL + D_CANULA + D_CON / 2]) rotate([0, 90, 0]) bend_pipe(ed=D_CANULA, id=0, r=D_CANULA, quarter=true);

                //Spacing canula, goes through the hemisphere
                translate([0, 0, WALL + D_CANULA + D_CON / 2]) cylinder(d=D_CANULA, h=2 * S_CANULA);
            }
        }
    }
    else
    {

        //Flat bottom, no port at the bottom
        if (port_bot == false)
        {
            //Bottom, flat
            cylinder(d=ed, h=WALL);
            translate([0, 0, WALL]) tube(ed=ed, id=id, h=id / 2);

            //Calculation of the volume
            h_vol = h - 2 * WALL - id / 2;
            volume = PI * pow((id / 2), 2) * h_vol + 4 / 3 * PI * pow((id / 2), 3);
            echo(volume / 1000);
        }
        //Flat bottom, port at the bottom
        else
        {

            translate([0, 0, id / 2 + WALL - (id / 2 + WALL - (WALL + D_CANULA + D_CON / 2 + S_CANULA))]) tube(ed=ed, id=id, h=id / 2 + WALL - (WALL + D_CANULA + D_CON / 2 + S_CANULA));

            difference()
            {
                cylinder(d=ed, h=WALL + D_CANULA + D_CON / 2 + S_CANULA);
                echo("Height of the flat bottom:", WALL + D_CANULA + D_CON / 2 + S_CANULA);

                if (adaptor_bot == true)
                {
                    //Adaptor tube
                    translate([0, -ed / 2 + L_CON_BOT, D_CON / 2 + WALL]) rotate([90, 0, 0]) cylinder(d=D_CON, h=L_CON_BOT);
                }

                //Perpendicular tube, canula
                translate([0, -D_CANULA, D_CON / 2 + WALL]) rotate([90, 0, 0]) cylinder(d=D_CANULA, h=ed / 2);
                echo("Reactor, height for bottom port:", D_CON / 2 + WALL);

                //Bend pipe, canula
                translate([0, -D_CANULA, WALL + D_CANULA + D_CON / 2]) rotate([0, 90, 0]) bend_pipe(ed=D_CANULA, id=0, r=D_CANULA, quarter=true);

                //Spacing canula
                translate([0, 0, WALL + D_CANULA + D_CON / 2]) cylinder(d=D_CANULA, h=S_CANULA);
            }
        }

    }

    if (open == false)
    {
        //Top input at the top of the reactor
        if (adaptor_top == true)
        {
            //Roof, hemisphere inside
            difference()
            {
                //Main cylinder of the roof
                translate([0, 0, h - id / 2 - WALL]) cylinder(d=ed, h=id / 2 + WALL);

                //Sphere for the curved shape of the roof
                translate([0, 0, h - id / 2 - WALL]) sphere(d=id);

                //Hole in the roof
                translate([0, 0, h - 2 * WALL]) cylinder(d=D_CON, h= 2 * WALL);

            }

            //Top connector
            difference()
            {%
                translate([0, 0, h]) cylinder(d=D_CON + 2 * WALL_CON, h=L_CON);
                translate([0, 0, h]) cylinder(d=D_CON, h=L_CON);
            }
        }

        else
        {
            //Roof, hemisphere inside
            difference()
            {
                //Main cylinder of the roof
                translate([0, 0, h - id / 2 - WALL]) cylinder(d=ed, h=id / 2 + WALL);

                //Sphere for the curved shape of the roof
                translate([0, 0, h - id / 2 - WALL]) sphere(d=id);
            }
        }
    }
    else
    {
        translate([0, 0, h - id / 2 - WALL]) tube(ed=ed, id=id, h=id / 2 + WALL);
    }
}

//More segments, higher definition
$fn=100;

reactor(ed=40, id=28, h=50, flat_bot=true, open=true, adaptor_top=true, port_bot=true, adaptor_bot=true);
